from dataclasses import dataclass

from gumpython.inputs import Input


@dataclass
class Flag:
    name: str
    type: str
    _value: Input = None
    _display_name: str = None

    @property
    def flag(self):
        return f"--{self.name}"

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value: Input):
        if value.is_valid():
            self._value = value

    @property
    def display_name(self):
        return self.name

    def get_command(self):
        return f"{self.flag}={self.value.compile()}"

    def __hash__(self):
        return hash((self.name,))


@dataclass
class SubFlag(Flag):
    sub_name: str = None
    separator: str = None

    @property
    def flag(self):
        return f"--{self.name}{self.separator}{self.sub_name}"

    @property
    def display_name(self):
        return f"{self.name} {self.sub_name}"

    def __hash__(self):
        return hash((self.name, self.sub_name))



from typing import List

from gumpython.arguments import ChooseArgument
from gumpython.exceptions import ChooseArgumentError
from gumpython.utils import (
    get_alignment_object,
    get_border_object,
    get_color_object,
    get_integer_input_object,
    get_position_object,
    get_string_input_object,
)

from .command import GumCommand


class Choose(GumCommand):
    def __init__(self, item_list: List[str]):
        super().__init__()
        self.argument = "choose"
        self.item_list = item_list
        self._validate_item_list()
        self.arguments = ChooseArgument()

    def _validate_item_list(self):
        if not isinstance(self.item_list, list):
            raise ChooseArgumentError(
                f"'list' expected for item list argument but '{type(self.item_list).__name__}' is given."
            )
        for item in self.item_list:
            if not isinstance(item, str):
                raise ChooseArgumentError(f"{item} is not a string value.")
        return True

    def _compile_command(self):
        super(Choose, self)._compile_command()
        self.command.extend(self.item_list)

    def cursor(self, cursor_text: str, prefix: str = None):
        cursor_arg = self.arguments.cursor
        self._add_to_flag_set(
            cursor_arg.cursor,
            get_string_input_object(
                cursor_text, cursor_arg.cursor.display_name
            ),
        )
        if prefix:
            self._add_to_flag_set(
                cursor_arg.prefix,
                get_string_input_object(
                    prefix, cursor_arg.prefix.display_name
                ),
            )
        return self

    def cursor_align(
        self, alignment: str, margin: tuple = None, padding: tuple = None
    ):
        cursor = self.arguments.cursor
        self._add_to_flag_set(
            cursor.align,
            get_alignment_object(alignment, cursor.align.display_name),
        )
        if margin:
            self._add_to_flag_set(
                cursor.margin,
                get_position_object(margin, cursor.margin.display_name),
            )
        if padding:
            self._add_to_flag_set(
                cursor.padding,
                get_position_object(padding, cursor.padding.display_name),
            )
        return self

    def cursor_color(
        self, foreground_color: str, background_color: str = None
    ):
        cursor = self.arguments.cursor
        self._add_to_flag_set(
            cursor.foreground, get_color_object(foreground_color)
        )
        if background_color:
            self._add_to_flag_set(
                cursor.background, get_color_object(background_color)
            )
        return self

    def cursor_style(
        self,
        bold: bool = False,
        italic: bool = False,
        faint: bool = False,
        underline: bool = False,
        strikethrough: bool = False,
    ):
        cursor = self.arguments.cursor
        if bold:
            self._add_to_flag_set(cursor.bold, None)
        if italic:
            self._add_to_flag_set(cursor.italic, None)
        if faint:
            self._add_to_flag_set(cursor.faint, None)
        if underline:
            self._add_to_flag_set(cursor.underline, None)
        if strikethrough:
            self._add_to_flag_set(cursor.strikethrough, None)
        return self

    def cursor_border(
        self,
        style: str,
        foreground_color: str = None,
        background_color: str = None,
    ):
        cursor = self.arguments.cursor
        self._add_to_flag_set(cursor.border, get_border_object(style))
        if foreground_color:
            self._add_to_flag_set(
                cursor.border_foreground, get_color_object(foreground_color)
            )
        if background_color:
            self._add_to_flag_set(
                cursor.border_background, get_color_object(background_color)
            )
        return self

    def size(self, width: int = None, height: int = None):
        cursor = self.arguments.cursor
        if width:
            self._add_to_flag_set(
                cursor.width,
                get_integer_input_object(width, cursor.width.display_name),
            )
        if height:
            self._add_to_flag_set(
                cursor.height,
                get_integer_input_object(height, cursor.height.display_name),
            )
        return self



from gumpython.exceptions import ColorInputError
from gumpython.inputs import (
    Alignment,
    Border,
    Color,
    IntegerInput,
    Position,
    StringInput,
)


def get_color_object(value):
    if isinstance(value, str):
        return Color(hex_code=value)
    if isinstance(value, tuple):
        return Color.from_rgb(value)
    raise ColorInputError(
        f"Expected 'str' or 'tuple' input for color but '{type(value).__name__}' given."
    )


def get_border_object(border_type):
    return Border(border_type)


def get_string_input_object(value, flag_name):
    return StringInput(value, flag_name)


def get_integer_input_object(value, flag_name):
    return IntegerInput(value, flag_name)


def get_alignment_object(value, flag_name):
    return Alignment(value, flag_name)


def get_position_object(value, flag_name):
    return Position(value, flag_name)



from gumpython.constants import FlagTypeChoice
from gumpython.run_command import run_command


class GumCommand:
    def __init__(self):
        self.command = None
        self.argument = None
        self.__flag_set = set()

    def run(self):
        self._compile_command()
        run_command(self.command)

    def _add_to_flag_set(self, flag, value):
        if flag.type != FlagTypeChoice.BOOL:
            flag.value = value
        self.__flag_set.add(flag)

    def _compile_command(self):
        self.command = ["gum", self.argument]
        for flag in self.__flag_set:
            if flag.type == FlagTypeChoice.BOOL:
                self.command.append(flag.flag)
            else:
                self.command.append(flag.get_command())

    def __str__(self):
        return " ".join(self.command)



from gumpython.commands import Choose, Style
from gumpython.inputs.help_inputs import alignment_types as alignment
from gumpython.inputs.help_inputs import border_types as border
from gumpython.inputs.help_inputs import color_names as color



from gumpython.flags import ChooseCursorFlag, StyleBorderFlag, StyleTextFlag


class Argument:
    pass


class StyleArgument(Argument):
    def __init__(self):
        self.border = StyleBorderFlag()
        self.text = StyleTextFlag()


class ChooseArgument(Argument):
    def __init__(self):
        self.cursor = ChooseCursorFlag()



from gumpython.constants import FlagSeparatorChoice, FlagTypeChoice

from .flags import Flag, SubFlag


class ChooseFlag:
    def __init__(self):
        self.cursor_flag_name = "cursor"


class ChooseCursorFlag(ChooseFlag):
    def __init__(self):
        super().__init__()
        # main
        self.cursor = Flag(name=self.cursor_flag_name, type=FlagTypeChoice.STR)
        self.prefix = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.STR,
            sub_name="prefix",
            separator=FlagSeparatorChoice.HYPHEN,
        )

        # align
        self.align = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.ALIGN,
            sub_name="align",
            separator=FlagSeparatorChoice.DOT,
        )
        self.margin = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.POSITION,
            sub_name="margin",
            separator=FlagSeparatorChoice.DOT,
        )
        self.padding = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.POSITION,
            sub_name="padding",
            separator=FlagSeparatorChoice.DOT,
        )

        # color
        self.background = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="background",
            separator=FlagSeparatorChoice.DOT,
        )
        self.foreground = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="foreground",
            separator=FlagSeparatorChoice.DOT,
        )

        # style
        self.bold = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="bold",
            separator=FlagSeparatorChoice.DOT,
        )
        self.faint = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="faint",
            separator=FlagSeparatorChoice.DOT,
        )
        self.italic = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="italic",
            separator=FlagSeparatorChoice.DOT,
        )
        self.strikethrough = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="strikethrough",
            separator=FlagSeparatorChoice.DOT,
        )
        self.underline = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="underline",
            separator=FlagSeparatorChoice.DOT,
        )

        # border
        self.border = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.BORDER,
            sub_name="border",
            separator=FlagSeparatorChoice.DOT,
        )
        self.border_background = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="border-background",
            separator=FlagSeparatorChoice.DOT,
        )
        self.border_foreground = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="border-foreground",
            separator=FlagSeparatorChoice.DOT,
        )

        # size
        self.height = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.INT,
            sub_name="height",
            separator=FlagSeparatorChoice.DOT,
        )
        self.width = SubFlag(
            name=self.cursor_flag_name,
            type=FlagTypeChoice.INT,
            sub_name="width",
            separator=FlagSeparatorChoice.DOT,
        )



from typing import List, Union

from gumpython.arguments import StyleArgument
from gumpython.exceptions import StyleArgumentError
from gumpython.utils import (
    get_alignment_object,
    get_border_object,
    get_color_object,
    get_integer_input_object,
    get_position_object,
)

from .command import GumCommand


class Style(GumCommand):
    def __init__(self, text: Union[str, List[str]]):
        super().__init__()
        self.argument = "style"
        self.text = text
        self._validate_text_argument()
        self.arguments = StyleArgument()

    def _compile_command(self):
        super(Style, self)._compile_command()
        if isinstance(self.text, str):
            self.command.append(self.text)
        if isinstance(self.text, list):
            self.command.extend(self.text)

    def _validate_text_argument(self):
        if isinstance(self.text, str):
            return True
        if isinstance(self.text, list):
            for t in self.text:
                if not isinstance(t, str):
                    raise StyleArgumentError(f"{t} is not a string.")
            return True
        raise StyleArgumentError(
            f"Expected 'str' or List[str] but '{type(self.text).__name__}' is given."
        )

    def border(
        self,
        style: str = None,
        foreground_color: Union[str, tuple] = None,
        background_color: Union[str, tuple] = None,
    ):
        border = self.arguments.border
        if style:
            self._add_to_flag_set(border.style, get_border_object(style))
        if foreground_color:
            self._add_to_flag_set(
                border.foreground_color, get_color_object(foreground_color)
            )
        if background_color:
            self._add_to_flag_set(
                border.background_color, get_color_object(background_color)
            )
        return self

    def align(
        self, alignment: str, margin: tuple = None, padding: tuple = None
    ):
        text = self.arguments.text
        self._add_to_flag_set(
            text.align,
            get_alignment_object(alignment, text.align.display_name),
        )
        if margin:
            self._add_to_flag_set(
                text.margin,
                get_position_object(margin, text.margin.display_name),
            )
        if padding:
            self._add_to_flag_set(
                text.padding,
                get_position_object(padding, text.padding.display_name),
            )
        return self

    def text_color(self, foreground, background=None):
        self._add_to_flag_set(
            self.arguments.text.foreground_color, get_color_object(foreground)
        )
        if background:
            self._add_to_flag_set(
                self.arguments.text.background_color,
                get_color_object(background),
            )
        return self

    def text_font(
        self,
        bold: bool = False,
        faint: bool = False,
        italic: bool = False,
        underline: bool = False,
        strikethrough: bool = False,
    ):
        text_arg = self.arguments.text
        if bold:
            self._add_to_flag_set(text_arg.bold, None)
        if faint:
            self._add_to_flag_set(text_arg.faint, None)
        if italic:
            self._add_to_flag_set(text_arg.italic, None)
        if underline:
            self._add_to_flag_set(text_arg.underline, None)
        if strikethrough:
            self._add_to_flag_set(text_arg.strikethrough, None)
        return self

    def size(self, width: int, height: int = None):
        text_arg = self.arguments.text
        self._add_to_flag_set(
            text_arg.width,
            get_integer_input_object(width, text_arg.width.display_name),
        )
        if height:
            self._add_to_flag_set(
                text_arg.height,
                get_integer_input_object(height, text_arg.height.display_name),
            )
        return self
